@@ -16,6 +16,7 @@
 # under the License.
 import json
 import logging
+import os
 import subprocess
 import sys
 from pathlib import Path
@@ -39,20 +40,27 @@ class SeleniumManager:
 
         :Returns: The Selenium Manager executable location
         """
-        platform = sys.platform
 
-        dirs = {
-            "darwin": "macos",
-            "win32": "windows",
-            "cygwin": "windows",
-        }
+        if os.getenv("SE_MANAGER_PATH"):
+            path = os.getenv("SE_MANAGER_PATH")
+        else:
+            platform = sys.platform
 
-        directory = dirs.get(platform) if dirs.get(platform) else platform
+            dirs = {
+                "darwin": "macos",
+                "win32": "windows",
+                "cygwin": "windows",
+            }
 
-        file = "selenium-manager.exe" if directory == "windows" else "selenium-manager"
+            directory = dirs.get(platform) if dirs.get(platform) else platform
+            file = "selenium-manager.exe" if directory == "windows" else "selenium-manager"
 
-        path = Path(__file__).parent.joinpath(directory, file)
+            path = Path(__file__).parent.joinpath(directory, file)
 
+        if not path.is_file() and os.environ["CONDA_PREFIX"]:
+            # conda has a separate package selenium-manager, installs in bin
+            path = Path(os.path.join(os.environ["CONDA_PREFIX"], "bin", file))
+            logger.debug(f"Conda environment detected, using `{path}`")
         if not path.is_file():
             raise WebDriverException(f"Unable to obtain working Selenium Manager binary; {path}")
 
@@ -61,8 +69,8 @@ class SeleniumManager:
         return path
 
     def driver_location(self, options: BaseOptions) -> str:
-        """
-        Determines the path of the correct driver.
+        """Determines the path of the correct driver.
+
         :Args:
          - browser: which browser to get the driver path for.
         :Returns: The driver path to use
@@ -93,18 +101,16 @@ class SeleniumManager:
         driver_path = output["driver_path"]
         logger.debug(f"Using driver at: {driver_path}")
 
-        try:
+        if hasattr(options.__class__, "binary_location"):
             options.binary_location = browser_path
-            options.browser_version = None  # geckodriver complains if this dev / nightly, etc
-        except AttributeError:
-            pass  # do not set on options classes that do not support it
+            options.browser_version = None  # if we have the binary location we no longer need the version
 
         return driver_path
 
     @staticmethod
     def run(args: List[str]) -> dict:
-        """
-        Executes the Selenium Manager Binary.
+        """Executes the Selenium Manager Binary.
+
         :Args:
          - args: the components of the command being executed.
         :Returns: The log string containing the driver location.
@@ -118,11 +124,9 @@ class SeleniumManager:
         logger.debug(f"Executing process: {command}")
         try:
             if sys.platform == "win32":
-                completed_proc = subprocess.run(
-                    args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, creationflags=subprocess.CREATE_NO_WINDOW
-                )
+                completed_proc = subprocess.run(args, capture_output=True, creationflags=subprocess.CREATE_NO_WINDOW)
             else:
-                completed_proc = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                completed_proc = subprocess.run(args, capture_output=True)
             stdout = completed_proc.stdout.decode("utf-8").rstrip("\n")
             stderr = completed_proc.stderr.decode("utf-8").rstrip("\n")
             output = json.loads(stdout)
